module CreateProj
  module Creator
    # Class for creating Ruby project
    class RubyCreator < Creator
      def initialize(*args)
        super(*args)
        @precommit_template = 'lint-pre-commit'
        @precommit_options = { linter: 'rubocop', file_ext: '.rb' }
        @gitignore_files = %w(.ruby-gemset .ruby-version .bundle
                              Gemfile.lock doc .yardoc)

        @gems_to_install = {
          'rubocop' => '0.28', 'rspec' => '3.0', 'yard' => '0.8'
        }
        @default_ruby_version = '2.0.0'
      end

      # Creates a namespaced gemset name to avoid overwriting of gemsets
      #
      # @example Get gemset name if project name is test
      #   gemset_name #=> 'test-createproj'
      #
      # @return [String] that has unique integer name
      def gemset_name
        "#{@name}-createproj"
      end

      # Write the .ruby-gemset and .ruby-version rvm files
      #
      # @todo - make it so can specify ruby version
      #
      # @example
      #   write_rvm_files #=> # writes the two files
      #
      # @return Nothing
      def write_rvm_files
        # must have both .ruby-gemset and .ruby-version file
        File.open('.ruby-gemset', 'w+') do |f|
          f.write(gemset_name)
        end

        # @todo - make this an option

        File.open('.ruby-version', 'w+') do |f|
          f.write(@default_ruby_version)
        end
      end

      # Create an rvm gemset with the project name and
      # creates file dictating gemset

      #
      # @example Create rvm sandbox for project named test
      #   install_sandbox #=> Executes `rvm gemset create test`
      #
      # @return Nothing
      def install_sandbox
        RVM.gemset_create(gemset_name)

        write_rvm_files
      end

      # Installs dependencies in the new sandbox
      #
      # @example Write rubocop to a gemfile and prompt `bundle install`
      #   install_dependencies #=> rubocop written in Gemfile
      #
      # @return Nothing
      def install_dependencies
        File.open('Gemfile', 'w+') do |f|
          f.write("source 'https://rubygems.org'\n\n")
          @gems_to_install.each do |k, v|
            f.write("gem '#{k}', '~> #{v}'\n")
          end
        end

        command = 'Enter the directory and run bundle install.'
        puts command
      end
    end
  end
end
